from scurl.mozilla_url_parse cimport (ParseStandardURL, ParseFileURL, ParseMailtoURL,
                                      ParseFileSystemURL, ParsePathURL, ExtractScheme,
                                      Parsed, Component)
from scurl.chromium_gurl cimport GURL
from scurl.chromium_url_constant cimport kFileScheme, kFileSystemScheme, kMailToScheme
from scurl.chromium_url_util_internal cimport CompareSchemeComponent
from scurl.chromium_url_util cimport IsStandard
from scurl.scurl_canonicalize_helper cimport canonicalize_component

import six
from six.moves.urllib.parse import urlsplit as stdlib_urlsplit
from six.moves.urllib.parse import urljoin as stdlib_urljoin
from six.moves.urllib.parse import urlunsplit as stdlib_urlunsplit
from six.moves.urllib.parse import urlparse as stdlib_urlparse
from six.moves.urllib.parse import urlunparse as stdlib_urlunparse

from libcpp.string cimport string
from libcpp cimport bool


cdef char * uses_params[15]
uses_params[:] = ['', 'ftp', 'hdl',
                   'prospero', 'http', 'imap',
                   'https', 'shttp', 'rtsp',
                   'rtspu', 'sip', 'sips',
                   'mms', 'sftp', 'tel']

cdef bytes slice_component(char * url, Component comp):
    if comp.len <= 0:
        return b""

    return url[comp.begin:comp.begin + comp.len]

cdef bytes build_netloc(char * url, Parsed parsed):
    if parsed.host.len <= 0:
        return b""

    # Nothing at all
    elif parsed.username.len <= 0 and parsed.password.len <= 0 and parsed.port.len <= 0:
        return url[parsed.host.begin: parsed.host.begin + parsed.host.len]

    # Only port
    elif parsed.username.len <= 0 and parsed.password.len <= 0 and parsed.port.len > 0:
        return url[parsed.host.begin: parsed.host.begin + parsed.host.len + 1 + parsed.port.len]

    # Only username
    elif parsed.username.len > 0 and parsed.password.len <= 0 and parsed.port.len <= 0:
        return url[parsed.username.begin: parsed.username.begin + parsed.host.len + 1 + parsed.username.len]

    # Username + password
    elif parsed.username.len > 0 and parsed.password.len > 0 and parsed.port.len <= 0:
        return url[parsed.username.begin: parsed.username.begin + parsed.host.len + 2 + parsed.username.len + parsed.password.len]

    # Username + port
    elif parsed.username.len > 0 and parsed.password.len <= 0 and parsed.port.len > 0:
        return url[parsed.username.begin: parsed.username.begin + parsed.host.len + 2 + parsed.username.len + parsed.port.len]

    # Username + port + password
    elif parsed.username.len > 0 and parsed.password.len > 0 and parsed.port.len > 0:
        return url[parsed.username.begin: parsed.username.begin + parsed.host.len + 3 + parsed.port.len  + parsed.username.len  + parsed.password.len]

    else:
        raise ValueError


cdef char * unicode_handling(str):
    """
    This function handles the unicode string and converts it to bytes
    which enables functions to receive unicode-type url as the input
    """
    cdef bytes bytes_str
    if isinstance(str, unicode):
        bytes_str = <bytes>(<unicode>str).encode('utf8')
    else:
        bytes_str = <bytes>str
    return bytes_str

cdef void parse_input_url(char * url, Component url_scheme, Parsed * parsed):
    """
    This function parses the input url using GURL url_parse
    """
    if CompareSchemeComponent(url, url_scheme, kFileScheme):
        ParseFileURL(url, len(url), parsed)
    elif CompareSchemeComponent(url, url_scheme, kFileSystemScheme):
        ParseFileSystemURL(url, len(url), parsed)
    elif IsStandard(url, url_scheme):
        ParseStandardURL(url, len(url), parsed)
    elif CompareSchemeComponent(url, url_scheme, kMailToScheme):
        """
        Discuss: Is this correct?
        """
        ParseMailtoURL(url, len(url), parsed)
    else:
        """
        TODO:
        trim or not to trim?
        """
        ParsePathURL(url, len(url), True, parsed)

# https://github.com/python/cpython/blob/master/Lib/urllib/parse.py
cdef object _splitparams(string path):
    """
    this function can be modified to enhance the performance?
    """
    cdef char slash_char = b'/'
    cdef string slash_string = b'/'
    cdef string semcol = b';'
    cdef int i

    if path.find(slash_string) != -1:
        i = path.find(semcol, path.rfind(slash_char))
        if i < 0:
            return path, b''
    else:
        i = path.find(semcol)
    return path.substr(0, i), path.substr(i + 1)


cdef class _NetlocResultMixinBase(object):
    """Shared methods for the parsed result objects containing a netloc element"""
    __slots__ = ()

    @property
    def username(self):
        return self._userinfo[0]

    @property
    def password(self):
        return self._userinfo[1]

    @property
    def hostname(self):
        hostname = self._hostinfo[0]
        if not hostname:
            return None
        # Scoped IPv6 address may have zone info, which must not be lowercased
        # like http://[fe80::822a:a8ff:fe49:470c%tESt]:1234/keys
        separator = '%' if isinstance(hostname, str) else b'%'
        hostname, percent, zone = hostname.partition(separator)
        return hostname.lower() + percent + zone

    @property
    def port(self):
        port = self._hostinfo[1]
        if port is not None:
            try:
                port = int(port, 10)
            except ValueError:
                message = 'Port could not be cast to integer value as {}'.format(port)
                raise ValueError(message) from None
            if not ( 0 <= port <= 65535):
                raise ValueError("Port out of range 0-65535")
        return port


cdef class _NetlocResultMixinStr(_NetlocResultMixinBase):
    __slots__ = ()

    @property
    def _userinfo(self):
        netloc = self[1]
        char_at, char_colon = '@', ':'
        if isinstance(netloc, bytes):
            char_at, char_colon = b'@', b':'

        userinfo, have_info, hostinfo = netloc.rpartition(char_at)
        if have_info:
            username, have_password, password = userinfo.partition(char_colon)
            if not have_password:
                password = None
        else:
            username = password = None
        return username, password

    @property
    def _hostinfo(self):
        netloc = self[1]
        char_at, char_leftsquare, char_rightsquare, char_colon = '@', '[', ']', ':'
        if isinstance(netloc, bytes):
            char_at, char_leftsquare, char_rightsquare, char_colon = b'@', b'[', b']', b':'

        _, _, hostinfo = netloc.rpartition(char_at)
        _, have_open_br, bracketed = hostinfo.partition(char_leftsquare)
        if have_open_br:
            hostname, _, port = bracketed.partition(char_rightsquare)
            _, _, port = port.partition(char_colon)
        else:
            hostname, _, port = hostinfo.partition(char_colon)
        if not port:
            port = None
        return hostname, port


cdef class UrlsplitResultAttribute(_NetlocResultMixinStr):
    __slots__ = ()

    @property
    def scheme(self):
        return self[0]

    @property
    def netloc(self):
        return self[1]

    @property
    def path(self):
        return self[2]

    @property
    def query(self):
        return self[3]

    @property
    def fragment(self):
        return self[4]


cdef class UrlparseResultAttribute(UrlsplitResultAttribute):
    __slots__ = ()

    @property
    def path(self):
        return self[2]

    @property
    def params(self):
        return self[3]

    @property
    def query(self):
        return self[4]

    @property
    def fragment(self):
        return self[5]


class SplitResultNamedTuple(tuple, UrlsplitResultAttribute):
    __slots__ = ()

    def __new__(cls, char * url, input_scheme, decode=False):

        cdef Parsed parsed
        cdef Component url_scheme

        if not ExtractScheme(url, len(url), &url_scheme):
            original_url = url.decode('utf-8') if decode else url
            return stdlib_urlsplit(original_url, input_scheme)

        parse_input_url(url, url_scheme, &parsed)

        scheme, netloc, path, query, ref = (slice_component(url, parsed.scheme).lower(),
                                            build_netloc(url, parsed),
                                            slice_component(url, parsed.path),
                                            slice_component(url, parsed.query),
                                            slice_component(url, parsed.ref))

        if not scheme and input_scheme:
            scheme = input_scheme.encode('utf-8')

        if decode:
            return tuple.__new__(cls, (
                <unicode>scheme.decode('utf-8'),
                <unicode>netloc.decode('utf-8'),
                <unicode>path.decode('utf-8'),
                <unicode>query.decode('utf-8'),
                <unicode>ref.decode('utf-8')
            ))

        return tuple.__new__(cls, (scheme, netloc, path, query, ref))

    def geturl(self):
        return stdlib_urlunsplit(self)


class ParsedResultNamedTuple(tuple, UrlparseResultAttribute):
    __slots__ = ()

    def __new__(cls, char * url, input_scheme,
                canonicalize, decode=False):

        cdef Parsed parsed
        cdef Component url_scheme

        if not ExtractScheme(url, len(url), &url_scheme):
            original_url = url.decode('utf-8') if decode else url
            return stdlib_urlparse(original_url, input_scheme)

        parse_input_url(url, url_scheme, &parsed)

        scheme, netloc, path, query, ref = (slice_component(url, parsed.scheme).lower(),
                                            build_netloc(url, parsed),
                                            slice_component(url, parsed.path),
                                            slice_component(url, parsed.query),
                                            slice_component(url, parsed.ref))
        if not scheme and input_scheme:
            scheme = input_scheme.encode('utf-8')

        cdef bool in_uses_params = False
        for param in uses_params:
            if param == scheme:
                in_uses_params = True
        if in_uses_params and b';' in path:
            path, params = _splitparams(path)
        else:
            params = b''

        # if canonicalize is set to true, then we will need to convert it to unicode
        if decode or canonicalize:
            return tuple.__new__(cls, (
                <unicode>scheme.decode('utf-8'),
                <unicode>netloc.decode('utf-8'),
                <unicode>path.decode('utf-8'),
                <unicode>params.decode('utf-8'),
                <unicode>query.decode('utf-8'),
                <unicode>ref.decode('utf-8')
            ))

        return tuple.__new__(cls, (scheme, netloc, path, params, query, ref))

    def geturl(self):
        return stdlib_urlunparse(self)


cpdef urlparse(url, scheme='', bool allow_fragments=True, bool canonicalize=False):
    """
    This function intends to replace urlparse from urllib
    using urlsplit function from scurl itself.
    Can this function be further enhanced?
    """
    decode = not isinstance(url, bytes)
    url = unicode_handling(url)
    return ParsedResultNamedTuple.__new__(ParsedResultNamedTuple, url, scheme,
                                          canonicalize, decode)

cpdef urlsplit(url, scheme='', bool allow_fragments=True):
    """
    This function intends to replace urljoin from urllib,
    which uses Urlparse class from GURL Chromium
    """
    decode = not isinstance(url, bytes)
    url = unicode_handling(url)
    return SplitResultNamedTuple.__new__(SplitResultNamedTuple, url, scheme, decode)

cpdef urljoin(base, url, bool allow_fragments=True):
    """
    This function intends to replace urljoin from urllib,
    which uses Resolve function from class GURL of GURL chromium
    """
    str_input = isinstance(base, str)
    if isinstance(url, str) != str_input:
        raise TypeError("Cannot mix str and non-str arguments")

    decode = not (isinstance(base, bytes) and isinstance(url, bytes))
    if allow_fragments and base:
        base, url = unicode_handling(base), unicode_handling(url)
        GURL_container = new GURL(base)
        # GURL will mark urls such as #, http:/// as invalid
        if not GURL_container.is_valid():
            fallback = stdlib_urljoin(base, url, allow_fragments=allow_fragments)
            if decode:
                return fallback.decode('utf-8')
            return fallback

        joined_url = GURL_container.Resolve(url).spec()

        if decode:
            return joined_url.decode('utf-8')
        return joined_url

    return stdlib_urljoin(base, url, allow_fragments=allow_fragments)



from libcpp.string cimport string
from libcpp cimport bool
from mozilla_url_parse cimport Component, Parsed


cdef extern from "../third_party/chromium/url/gurl.h":
    cdef cppclass GURL:
        GURL()
        GURL(const string & url_string)
        GURL(const char * canonical_spec,
             size_t canonical_spec_len,
             const Parsed parsed,
             bool is_valid)

        bool is_valid()
        bool is_empty()
        bool IsStandard()
        string spec()
        GURL Resolve(const string & relative)
        string possibly_invalid_spec()

        bool has_scheme()
        bool has_username()
        bool has_password()
        bool has_host()
        bool has_port()
        bool has_path()
        bool has_query()
        bool has_ref()

        string scheme()
        string username()
        string password()
        string host()
        string port()
        string path()
        string query()
        string ref()

        Parsed parsed_for_possibly_invalid_spec()
        # GURL ReplaceComponents(const Replacements[char] replacements)



from libcpp.string cimport string
from libcpp cimport bool
from mozilla_url_parse cimport Component, Parsed


cdef extern from "../third_party/chromium/url/url_constants.h" namespace "url":

    extern const char kAboutBlankURL[];

    extern const char kAboutScheme[];
    extern const char kBlobScheme[];

    extern const char kContentScheme[];
    extern const char kDataScheme[];
    extern const char kFileScheme[];
    extern const char kFileSystemScheme[];
    extern const char kFtpScheme[];
    extern const char kGopherScheme[];
    extern const char kHttpScheme[];
    extern const char kHttpsScheme[];
    extern const char kJavaScriptScheme[];
    extern const char kMailToScheme[];
    extern const char kWsScheme[];
    extern const char kWssScheme[];

    extern const char kStandardSchemeSeparator[];



from scurl.mozilla_url_parse cimport Component
from scurl.chromium_url_canon_stdstring cimport StdStringCanonOutput
from scurl.chromium_url_canon cimport CanonicalizePath

from libcpp.string cimport string


cdef inline string canonicalize_component(char * url, Component parsed_comp):
    """
    This function canonicalizes the components of the urls
    Using Chromium GURL canonicalize func
    """
    cdef Component output_comp
    cdef string canonicalized_output = string()
    cdef StdStringCanonOutput * output = new StdStringCanonOutput(&canonicalized_output)
    # CanonicalizeQuery has different way of canonicalize encoded urls
    # so we will use canonicalizePath for now!
    # CanonicalizeQuery(query, query_comp, NULL, output, &out_query)
    is_valid = CanonicalizePath(url, parsed_comp, output, &output_comp)
    output.Complete()


    if canonicalized_output.length() > 0 and canonicalized_output[0] == "/":
        canonicalized_output = canonicalized_output.substr(1)

    return canonicalized_output



import six

if six.PY2:
    from urlparse import *
else:
    from urllib.parse import *
    # these cannot be imported in py34
    from urllib.parse import DefragResult, SplitResult, ParseResult


_original_urlsplit = urlsplit
_original_urljoin = urljoin
_original_urlparse = urlparse

from scurl.cgurl import urlsplit, urljoin, urlparse
"""
TODO: find some way to not import parse_url
"""
from scurl.canonicalize import canonicalize_url, parse_url



from scurl import urlparse

import string
import six
import logging
from six.moves.urllib.parse import (urlunsplit, urldefrag, urlencode,
                                    quote, parse_qsl, unquote)
from six.moves.urllib.parse import urlunparse as stdlib_urlunparse
from scurl.scurl_canonicalize_helper cimport canonicalize_component
from scurl.mozilla_url_parse cimport Component, MakeRange


logger = logging.getLogger('scurl')

# https://github.com/scrapy/w3lib/blob/master/w3lib/url.py
RFC3986_GEN_DELIMS = b':/?#[]@'
RFC3986_SUB_DELIMS = b"!$&'()*+,;="
RFC3986_RESERVED = RFC3986_GEN_DELIMS + RFC3986_SUB_DELIMS
RFC3986_UNRESERVED = (string.ascii_letters + string.digits + "-._~").encode('ascii')
EXTRA_SAFE_CHARS = b'|'  # see https://github.com/scrapy/w3lib/pull/25

_safe_chars = RFC3986_RESERVED + RFC3986_UNRESERVED + EXTRA_SAFE_CHARS + b'%'

if not six.PY2:
    from urllib.parse import _coerce_args, unquote_to_bytes

    def parse_qsl_to_bytes(qs, keep_blank_values=False):
        """Parse a query given as a string argument.

        Data are returned as a list of name, value pairs as bytes.

        Arguments:

        qs: percent-encoded query string to be parsed

        keep_blank_values: flag indicating whether blank values in
            percent-encoded queries should be treated as blank strings.  A
            true value indicates that blanks should be retained as blank
            strings.  The default false value indicates that blank values
            are to be ignored and treated as if they were  not included.

        """
        # This code is the same as Python3's parse_qsl()
        # (at https://hg.python.org/cpython/rev/c38ac7ab8d9a)
        # except for the unquote(s, encoding, errors) calls replaced
        # with unquote_to_bytes(s)
        qs, _coerce_result = _coerce_args(qs)
        pairs = [s2 for s1 in qs.split('&') for s2 in s1.split(';')]
        r = []
        for name_value in pairs:
            if not name_value:
                continue
            nv = name_value.split('=', 1)
            if len(nv) != 2:
                # Handle case of a control-name with no equal sign
                if keep_blank_values:
                    nv.append('')
                else:
                    continue
            if len(nv[1]) or keep_blank_values:
                name = nv[0].replace('+', ' ')
                name = unquote_to_bytes(name)
                name = _coerce_result(name)
                value = nv[1].replace('+', ' ')
                value = unquote_to_bytes(value)
                value = _coerce_result(value)
                r.append((name, value))
        return r

def _safe_ParseResult(parts, encoding='utf8'):
    """
    NOTE: This function is from w3lib. However, it has been modified
    to use functions from scurl instead!
    """
    if encoding is None:
        encoding = 'utf-8'

    # IDNA encoding can fail for too long labels (>63 characters)
    # or missing labels (e.g. http://.example.com)
    try:
        netloc = parts.netloc.encode('idna')
    except UnicodeError:
        netloc = parts.netloc

    # all the components from parts are string on py3 and unicode on py2
    query, fragment = parts.query, parts.fragment

    try:
        query, fragment = query.encode(encoding), fragment.encode(encoding)
    except UnicodeEncodeError as e:
        query, fragment = query.encode('utf-8'), fragment.encode('utf-8')
        logger.debug('Failed to encode query to the selected encoding!')

    cdef Component query_comp = MakeRange(0, len(query))
    cdef Component fragment_comp = MakeRange(0, len(fragment))

    query = canonicalize_component(query, query_comp)
    fragment = canonicalize_component(fragment, fragment_comp)

    return (
        to_native_str(parts.scheme),
        to_native_str(netloc),

        # default encoding for path component SHOULD be UTF-8
        to_native_str(parts.path),
        to_native_str(parts.params),

        # encoding of query and fragment follows page encoding
        # or form-charset (if known and passed)
        to_native_str(query, encoding),
        to_native_str(fragment, encoding)
    )

cpdef canonicalize_url(url, keep_blank_values=True, keep_fragments=False,
                     encoding=None):
    r"""Canonicalize the given url by applying the following procedures:

    - sort query arguments, first by key, then by value
    - percent encode paths ; non-ASCII characters are percent-encoded
      using UTF-8 (RFC-3986)
    - percent encode query arguments ; non-ASCII characters are percent-encoded
      using passed `encoding` (UTF-8 by default)
    - normalize all spaces (in query arguments) '+' (plus symbol)
    - normalize percent encodings case (%2f -> %2F)
    - remove query arguments with blank values (unless `keep_blank_values` is True)
    - remove fragments (unless `keep_fragments` is True)

    The url passed can be bytes or unicode, while the url returned is
    always a native str (bytes in Python 2, unicode in Python 3).

    >>> import w3lib.url
    >>>
    >>> # sorting query arguments
    >>> w3lib.url.canonicalize_url('http://www.example.com/do?c=3&b=5&b=2&a=50')
    'http://www.example.com/do?a=50&b=2&b=5&c=3'
    >>>
    >>> # UTF-8 conversion + percent-encoding of non-ASCII characters
    >>> w3lib.url.canonicalize_url(u'http://www.example.com/r\u00e9sum\u00e9')
    'http://www.example.com/r%C3%A9sum%C3%A9'
    >>>

    For more examples, see the tests in `tests/test_url.py`.

    NOTE: This function is from w3lib. However, it has been modified
    to use functions from scurl instead!
    """
    try:
        scheme, netloc, path, params, query, fragment = _safe_ParseResult(
            parse_url(url, encoding), encoding=encoding)
    except UnicodeEncodeError as e:
        scheme, netloc, path, params, query, fragment = _safe_ParseResult(
            parse_url(url, encoding), encoding='utf8')

    # 1. decode query-string as UTF-8 (or keep raw bytes),
    #    sort values,
    #    and percent-encode them back
    if six.PY2:
        keyvals = parse_qsl(query, keep_blank_values)
    else:
        # Python3's urllib.parse.parse_qsl does not work as wanted
        # for percent-encoded characters that do not match passed encoding,
        # they get lost.
        #
        # e.g., 'q=b%a3' becomes [('q', 'b\ufffd')]
        # (ie. with 'REPLACEMENT CHARACTER' (U+FFFD),
        #      instead of \xa3 that you get with Python2's parse_qsl)
        #
        # what we want here is to keep raw bytes, and percent encode them
        # so as to preserve whatever encoding what originally used.
        #
        # See https://tools.ietf.org/html/rfc3987#section-6.4:
        #
        # For example, it is possible to have a URI reference of
        # "http://www.example.org/r%E9sum%E9.xml#r%C3%A9sum%C3%A9", where the
        # document name is encoded in iso-8859-1 based on server settings, but
        # where the fragment identifier is encoded in UTF-8 according to
        # [XPointer]. The IRI corresponding to the above URI would be (in XML
        # notation)
        # "http://www.example.org/r%E9sum%E9.xml#r&#xE9;sum&#xE9;".
        # Similar considerations apply to query parts.  The functionality of
        # IRIs (namely, to be able to include non-ASCII characters) can only be
        # used if the query part is encoded in UTF-8.
        keyvals = parse_qsl_to_bytes(query, keep_blank_values)
    keyvals.sort()
    query = urlencode(keyvals)

    # 2. decode percent-encoded sequences in path as UTF-8 (or keep raw bytes)
    #    and percent-encode path again (this normalizes to upper-case %XX)
    uqp = _unquotepath(path)
    cdef Component path_component = Component(0, len(uqp))
    path = to_native_str(canonicalize_component(uqp, path_component)) or '/'

    fragment = '' if not keep_fragments else fragment

    # every part should be safe already
    return stdlib_urlunparse((scheme,
                              netloc.lower().rstrip(':'),
                              path,
                              params,
                              query,
                              fragment))

def parse_url(url, canonicalize_encoding='utf-8', encoding=None):
    """Return urlparsed url from the given argument (which could be an already
    parsed url)

    NOTE: This function is from w3lib. However, it has been modified
    to use functions from scurl instead!
    """
    if isinstance(url, tuple):
        return url

    if canonicalize_encoding is None:
        canonicalize_encoding = 'utf-8'
    return urlparse(to_unicode(url, encoding), canonicalize=True)

def _unquotepath(path):
    for reserved in ('2f', '2F', '3f', '3F'):
        path = path.replace('%' + reserved, '%25' + reserved.upper())

    if six.PY2:
        # in Python 2, '%a3' becomes '\xa3', which is what we want
        return unquote(path)
    else:
        # in Python 3,
        # standard lib's unquote() does not work for non-UTF-8
        # percent-escaped characters, they get lost.
        # e.g., '%a3' becomes 'REPLACEMENT CHARACTER' (U+FFFD)
        #
        # unquote_to_bytes() returns raw bytes instead
        return unquote_to_bytes(path)


# util funcs
# https://github.com/scrapy/w3lib/blob/master/w3lib/util.py

def to_unicode(text, encoding=None, errors='strict'):
    """Return the unicode representation of a bytes object `text`. If `text`
    is already an unicode object, return it as-is."""
    if isinstance(text, six.text_type):
        return text
    if not isinstance(text, (bytes, six.text_type)):
        raise TypeError('to_unicode must receive a bytes, str or unicode '
                        'object, got %s' % type(text).__name__)
    if encoding is None:
        encoding = 'utf-8'
    return text.decode(encoding, errors)

def to_bytes(text, encoding=None, errors='strict'):
    """Return the binary representation of `text`. If `text`
    is already a bytes object, return it as-is."""
    if isinstance(text, bytes):
        return text
    if not isinstance(text, six.string_types):
        raise TypeError('to_bytes must receive a unicode, str or bytes '
                        'object, got %s' % type(text).__name__)
    if encoding is None:
        encoding = 'utf-8'
    return text.encode(encoding, errors)

def to_native_str(text, encoding=None, errors='strict'):
    """ Return str representation of `text`
    (bytes in Python 2.x and unicode in Python 3.x). """
    if six.PY2:
        return to_bytes(text, encoding, errors)
    else:
        return to_unicode(text, encoding, errors)



from libcpp.string cimport string
from libcpp cimport bool
from mozilla_url_parse cimport Component, Parsed
from chromium_url_canon cimport CharsetConverter
from chromium_url_canon_stdstring cimport StdStringCanonOutput


cdef extern from "../third_party/chromium/url/url_util.h" namespace "url":
    cdef bool IsStandard(const char* spec, const Component& scheme)
    cdef bool Canonicalize(const char* spec,
                           int spec_len,
                           bool trim_path_end,
                           CharsetConverter* charset_converter,
                           StdStringCanonOutput* output,
                           Parsed* output_parsed)



from libcpp.string cimport string
from libcpp cimport bool
from mozilla_url_parse cimport Component, Parsed


cdef extern from "../third_party/chromium/url/url_util_internal.h" namespace "url":
    cdef bool CompareSchemeComponent(const char* spec,
                                     const Component& component,
                                     const char* compare_to)



from libcpp cimport bool


cdef extern from "../third_party/chromium/url/third_party/mozilla/url_parse.h" namespace "url":
    cdef cppclass Component:
        Component()
        Component(int b, int l)
        int begin
        int len

    cdef cppclass Parsed:
        int Length()
        Component scheme
        Component username
        Component password
        Component host
        Component port
        Component path
        Component query
        Component ref

    cdef Component MakeRange(int begin, int end)
    cdef void ParseStandardURL(const char* url, int url_len, Parsed* parsed)
    cdef void ParseFileURL(const char* url, int url_len, Parsed* parsed)
    cdef void ParseMailtoURL(const char* url, int url_len, Parsed* parsed)
    cdef void ParseFileSystemURL(const char* url, int url_len, Parsed* parsed)
    cdef void ParsePathURL(const char* url, int url_len, bool trim_path_end, Parsed* parsed)
    cdef bool ExtractScheme(const char* url, int url_len, Component* scheme)



from libcpp.string cimport string


cdef extern from "../third_party/chromium/url/url_canon_stdstring.h" namespace "url":
    cdef cppclass StdStringCanonOutput:
        StdStringCanonOutput()
        StdStringCanonOutput(string* str)
        void Complete()
        void Resize(int sz)



from setuptools.extension import Extension
from setuptools import setup, find_packages
import os
from os.path import splitext
import logging
import platform
from glob import glob

VERSION = "0.1.0"
ext_macros = []
logger = logging.getLogger('scurl')

try:
    from Cython.Compiler import Options as CythonOptions
except ImportError as e:
    logger.debug('Cython is not installed on your env.\
                  Please get the latest version of Cython and try again!')

if os.environ.get('CYTHON_TRACE'):
    if platform.python_implementation() != 'PyPy':
        # enable linetrace in Cython
        ext_macros.append(('CYTHON_TRACE', '1'))
        cython_defaults = CythonOptions.get_directive_defaults()
        cython_defaults['linetrace'] = True
        logger.warning('Warning: Enabling line tracing in Cython extension.\
                        This will make the performance of the library less effective!')

cgurl_ext_sources = [
    'scurl/cgurl.pyx',
    'third_party/chromium/base/callback_internal.cc',
    'third_party/chromium/base/at_exit.cc',
    'third_party/chromium/base/lazy_instance_helpers.cc',
    'third_party/chromium/base/strings/utf_string_conversion_utils.cc',
    'third_party/chromium/base/strings/string_piece.cc',
    'third_party/chromium/base/strings/string16.cc',
    'third_party/chromium/base/strings/string_util.cc',
    'third_party/chromium/base/strings/utf_string_conversions.cc',
    'third_party/chromium/base/strings/string_util_constants.cc',
    'third_party/chromium/base/third_party/icu/icu_utf.cc',
    'third_party/chromium/url/gurl.cc',
    'third_party/chromium/url/url_canon.cc',
    'third_party/chromium/url/url_canon_etc.cc',
    # 'third_party/chromium/url/url_canon_icu.cc',
    'third_party/chromium/url/url_canon_filesystemurl.cc',
    'third_party/chromium/url/url_canon_fileurl.cc',
    'third_party/chromium/url/url_canon_host.cc',
    'third_party/chromium/url/url_canon_internal.cc',
    'third_party/chromium/url/url_canon_ip.cc',
    'third_party/chromium/url/url_canon_mailtourl.cc',
    'third_party/chromium/url/url_canon_path.cc',
    'third_party/chromium/url/url_canon_pathurl.cc',
    'third_party/chromium/url/url_canon_query.cc',
    'third_party/chromium/url/url_canon_relative.cc',
    'third_party/chromium/url/url_canon_stdstring.cc',
    'third_party/chromium/url/url_canon_stdurl.cc',
    'third_party/chromium/url/url_constants.cc',
    'third_party/chromium/url/url_parse_file.cc',
    'third_party/chromium/url/url_util.cc',
    'third_party/chromium/url/third_party/mozilla/url_parse.cc',
]

canonicalize_ext_sources = [
    'scurl/canonicalize.pyx',
    'third_party/chromium/base/callback_internal.cc',
    'third_party/chromium/base/at_exit.cc',
    'third_party/chromium/base/lazy_instance_helpers.cc',
    'third_party/chromium/base/strings/utf_string_conversion_utils.cc',
    'third_party/chromium/base/strings/string_piece.cc',
    'third_party/chromium/base/strings/string16.cc',
    'third_party/chromium/base/strings/string_util.cc',
    'third_party/chromium/base/strings/utf_string_conversions.cc',
    'third_party/chromium/base/strings/string_util_constants.cc',
    'third_party/chromium/base/third_party/icu/icu_utf.cc',
    'third_party/chromium/url/url_canon.cc',
    'third_party/chromium/url/url_canon_path.cc',
    'third_party/chromium/url/url_canon_internal.cc',
    'third_party/chromium/url/url_canon_stdstring.cc',
]

extension = [
    Extension(
        name="scurl.cgurl",
        sources=cgurl_ext_sources,
        language="c++",
        extra_compile_args=["-std=gnu++14", "-I./third_party/chromium/",
                            "-fPIC", "-Ofast", "-pthread", "-w", '-DU_COMMON_IMPLEMENTATION'],
        extra_link_args=["-std=gnu++14", "-w"],
        include_dirs=['.'],
        define_macros=ext_macros
    ),
    Extension(
        name="scurl.canonicalize",
        sources=canonicalize_ext_sources,
        language="c++",
        extra_compile_args=["-std=gnu++14", "-I./third_party/chromium/",
                            "-fPIC", "-Ofast", "-pthread", "-w"],
        extra_link_args=["-std=gnu++14", "-w"],
        include_dirs=['.'],
        define_macros=ext_macros
    )
]


if not os.path.isfile("scurl/cgurl.cpp"):
    try:
        from Cython.Build import cythonize
        ext_modules = cythonize(extension, annotate=True)
    except ImportError:
        print("scurl/cgurl.cpp not found and Cython failed to run to recreate it. Please install/upgrade Cython and try again.")
        raise
else:
    ext_modules = extension
    ext_modules[0].sources[0] = "scurl/cgurl.cpp"
    ext_modules[1].sources[0] = "scurl/canonicalize.cpp"

try:
    import pypandoc
    long_description = pypandoc.convert('README.md', 'rst')
except ImportError:
    long_description = open('README.md').read()

setup(
    name="scurl",
    packages=find_packages(exclude=('tests', 'tests.*')),
    version=VERSION,
    description="",
    license="Apache License, Version 2.0",
    url="https://github.com/scrapy/scurl",
    keywords=["urlparse", "urlsplit", "urljoin", "url", "parser", "urlparser", "parsing", "gurl", "cython", "faster", "speed", "performance"],
    platforms='any',
    classifiers=[
        "Programming Language :: Python",
        'Programming Language :: Python :: 2',
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy',
        "Intended Audience :: Developers",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
        "Topic :: Software Development :: Libraries"
    ],
    install_requires=[
        'six>=1.5.2',
    ],
    long_description=long_description,
    ext_modules=ext_modules,
    include_package_data=True,
    zip_safe=False
)



from libcpp cimport bool
from mozilla_url_parse cimport Component, Parsed
from chromium_url_canon_stdstring cimport StdStringCanonOutput


cdef extern from "../third_party/chromium/url/url_canon.h" namespace "url":
    cdef cppclass CanonOutputT:
        pass

    cdef cppclass CharsetConverter:
        CharsetConverter()

    ctypedef CanonOutputT CanonOutput

    cdef bool CanonicalizePath(const char* spec,
                                const Component& path,
                                StdStringCanonOutput* output,
                                Component* out_path)
    cdef void CanonicalizeQuery(const char* spec,
                                const Component& query,
                                CharsetConverter* converter,
                                StdStringCanonOutput* output,
                                Component* out_query)
    cdef void CanonicalizeRef(const char* spec,
                                const Component& path,
                                StdStringCanonOutput* output,
                                Component* out_path)
